package com.discovergy.apiclient;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.Map;
import java.util.concurrent.ExecutionException;

import javax.net.ssl.HttpsURLConnection;

import com.github.scribejava.core.builder.ServiceBuilder;
import com.github.scribejava.core.model.OAuth1AccessToken;
import com.github.scribejava.core.model.OAuth1RequestToken;
import com.github.scribejava.core.model.OAuthRequest;
import com.github.scribejava.core.model.Response;
import com.github.scribejava.core.model.Verb;
import com.github.scribejava.core.oauth.OAuth10aService;

import flexjson.JSONDeserializer;

/**
 * This class is to be used as an example for accessing Discovergy API (api.discovergy.com/docs) programatically
 * Usage:
 *  - create new DiscovergyAPIClient
 *  - initialize (method initialize())
 *  - execute get request (method executeGetRequest() with already completed URL. This method can be easily adapted for POST or any other type of requests
 * 
 * @author sergiu
 *
 */
public class DiscovergyAPIClient {

	/**
	 * Discovergy User
	 */
	private final String user;
	
	/**
	 * Password for Discovergy User
	 */
	private final String password;
	
	/**
	 * Unique client id
	 */
	private final String clientId;

	private OAuth10aService authenticationService;
	private OAuth1AccessToken accessToken;

	private final JSONDeserializer<Map<String, String>> deserializer = new JSONDeserializer<>();

	public DiscovergyAPIClient(String user, String password, String clientId) {
		this.user = user;
		this.password = password;
		this.clientId = clientId;
	}

	public static void main(String[] args) throws Exception {
		DiscovergyAPIClient apiClient = new DiscovergyAPIClient("user", "password", "uniqueClientId");
		apiClient.initialize();
		String response = apiClient.executeGetRequest("https://api.discovergy.com/public/v1/meters");
		System.out.println(response);
	}

	public void initialize() throws Exception {
		Map<String, String> consumerTokenEntries = getConsumerToken();
		authenticationService = new ServiceBuilder().apiKey(consumerTokenEntries.get("key"))
				.apiSecret(consumerTokenEntries.get("secret")).build(new DiscovergyAPI(user, password));
		OAuth1RequestToken requestToken = authenticationService.getRequestToken();
		String authorizationURL = authenticationService.getAuthorizationUrl(requestToken);
		String verifier = authorize(authorizationURL);
		accessToken = authenticationService.getAccessToken(requestToken, verifier);
	}

	public String executeGetRequest(String url) throws InterruptedException, ExecutionException, IOException {
		OAuthRequest request = new OAuthRequest(Verb.GET, url);
		authenticationService.signRequest(accessToken, request);
		final Response response = authenticationService.execute(request);
		return response.getBody();
	}

	private Map<String, String> getConsumerToken() throws Exception {
		byte[] rawRequest = ("client=" + clientId).getBytes(Charset.forName("UTF8"));
		HttpsURLConnection connection = getConnection("https://api.discovergy.com/public/v1/oauth1/consumer_token",
				"POST", true, true);
		connection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded; charset=utf-8");
		connection.setRequestProperty("Content-Length", Integer.toString(rawRequest.length));
		connection.connect();
		connection.getOutputStream().write(rawRequest);
		connection.getOutputStream().flush();
		String content = getContent(connection.getInputStream());
		connection.disconnect();
		return deserializer.deserialize(content);
	}

	private String authorize(String authorizationURL) throws Exception {
		HttpsURLConnection connection = getConnection(authorizationURL, "GET", true, false);
		connection.connect();
		String content = getContent(connection.getInputStream());
		connection.disconnect();
		return content.substring(content.indexOf("=") + 1);
	}

	public HttpsURLConnection getConnection(String rawURL, String method, boolean doInput, boolean doOutput)
			throws IOException {
		URL url = new URL(rawURL);
		HttpsURLConnection connection = (HttpsURLConnection) url.openConnection();
		connection.setDoInput(doInput);
		connection.setDoOutput(doOutput);
		connection.setRequestMethod(method);
		connection.setRequestProperty("Accept", "*");
		connection.setInstanceFollowRedirects(false);
		connection.setRequestProperty("charset", "utf-8");
		connection.setUseCaches(false);
		return connection;
	}

	private String getContent(InputStream inputStream) throws IOException {
		StringBuilder content = new StringBuilder();
		try (BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream))) {
			String line = null;

			while ((line = reader.readLine()) != null) {
				content.append(line);
			}
		}
		return content.toString();
	}

}